import { PolymerElement, html } from '@polymer/polymer/polymer-element.js';

import { Annotation } from './annotation.js';

class CommentEditor extends PolymerElement {
  static get template() {
    return html`
    <style>
    :host {
      display:block;
    }
    .editor-row.no-title {
      margin-top: 0.75rem;
    }
    .actions {
      margin-top: 0.75rem;
      -webkit-display: -webkit-box;
      -webkit-display: -webkit-flex;
      display: -webkit-box;
      display: -ms-flexbox;
      display: flex;
      flex-direction: row;
      flex-wrap: nowrap;
      align-items: center;
      justify-content: flex-end;
    }
    .actions paper-button {
      flex: none;
    }
    paper-textarea {
        --paper-input-container-input: { background: #ddd; }
    }
    </style>

    <div class="editor-row">
      <paper-textarea id="commentInput" rows="10" value="{{comment.value}}" label="Comment" name="comment" autofocus=""></paper-textarea>
    </div>
    <div class="editor-row no-title">
      <paper-checkbox checked="{{private.value}}" id="privateInput" value="private" name="private">this is a private comment</paper-checkbox>
    </div>

    <div class="actions">
      <paper-button class="cancel_button" on-tap="cancel" disabled="">Cancel</paper-button>
      <paper-button on-tap="save" class="cancel_button" disabled\$="[[submitDisabled]]">save</paper-button>
    </div>
`;
  }

  static get properties() {
    return {
      comment: {
        type: Object,
        observer: '_handleChange'
      },
      private: Object,
      submitDisabled: {
        type: Boolean,
        value: true
      }
    }
  }

  constructor() {
    super();
  }

  _handleChange(newValue, oldValue) {
    this._checkSubmitDisabled();
    if (!newValue) { return }
    this.$.commentInput.validate();
  }

  connectedCallback() {
    super.connectedCallback();
    const commentInput = this.shadowRoot.getElementById("commentInput");
    commentInput.addEventListener('change', event => this._handleValueChange(event));
    const privateInput = this.shadowRoot.getElementById("privateInput");
    privateInput.addEventListener('change', event => this._handleValueChange(event));
  }

  disconnectedCallback() {
    super.disconnectedCallback();
    const commentInput = this.shadowRoot.getElementById("commentInput");
    commentInput.removeEventListener('change', event => this._handleValueChange(event));
    const privateInput = this.shadowRoot.getElementById("privateInput");
    privateInput.removeEventListener('change', event => this._handleValueChange(event));
  }

  setTargets(targets) {
    function realTarget(element) {
      if (element.orig) { return element.orig }
      return element
    }
    this.targets = targets.map(realTarget)
  }

  save() {
    this.dispatchEvent(new CustomEvent('saved', {
      bubbles: true,
      composed: true,
      detail: {
        ticket: Annotation.getNewSessionKey(),
        id: this.comment.annotationId,
        body: this.comment.value,
        targets: this.targets,
        visibility: this.private.value ? 'private' : 'public',
        action: Annotation.ActionTypes.COMMENT
      }
    }));
    this._reset();
    this.dispatchEvent(new CustomEvent('closed', { bubbles: true, composed: true }));
  }

  cancel() {
    this.dispatchEvent(new CustomEvent('cancelled', { bubbles: true, composed: true }));
    this._reset();
    this.dispatchEvent(new CustomEvent('closed', { bubbles: true, composed: true }));
  }

  _handleValueChange(event) {
    this[event.target.name].dirty = true;
  }

  _reset() {
    this.comment = {
      value: '',
      dirty: false
    }
    this.private = {
      value: false,
      dirty: false
    }
  }

  _checkSubmitDisabled() {
    this.submitDisabled = (
      this.comment.length > 0 &&
      !(this.private.dirty || this.comment.dirty)
    );
  }
}

customElements.define('comment-editor', CommentEditor);
