import { PolymerElement, html } from '@polymer/polymer/polymer-element.js';
import { Annotation } from './annotation.js';

class JoinEditor extends PolymerElement {
  static get template() {
    return html`
    <style>
      .page-title {
        @apply(--paper-font-display2);
      }

      .app-menu {
        --paper-menu-selected-item: {
          color: #fff;
          background: var(--light-primary-color);
        }
      }

      paper-menu a > *, paper-menu paper-item > *, paper-menu paper-icon-item > * {
        pointer-events: none;
      }

      @media (max-width: 600px) {
        .page-title {
          font-size: 24px!important;
        }
      }

      h1, h2, h3, h4, h5, h6 {
        font-family: "IM Fell English", serif;
        line-height: 1.2;
      }

      .btn-default:hover {
        color: #333;
        background-color: #e6e6e6;
        border-color: #adadad;
      }

      .btn-block {
        display: block;
        width: 100%;
        height: 24px;
      }

      .btn:not(.btn-raised), .btn:not(.btn-raised).btn-default, .input-group-btn .btn:not(.btn-raised), .input-group-btn .btn:not(.btn-raised).btn-default {
        color: rgba(0, 0, 0, 0.87);
      }

      .btn, .input-group-btn .btn {
        border: none;
        border-radius: 2px;
        margin: 5px 1px;
        text-transform: uppercase;
        letter-spacing: 0;
        will-change: box-shadow, transform;
        transition: box-shadow 0.2s cubic-bezier(0.4, 0, 1, 1), background-color 0.2s cubic-bezier(0.4, 0, 0.2, 1), color 0.2s cubic-bezier(0.4, 0, 0.2, 1);
        outline: 0;
        cursor: pointer;
        text-decoration: none;
        background: 0 0;
      }

      :host {
        display: block;
      }
      .actions {
        margin-top: 2rem;
        -webkit-display: -webkit-box;
        -webkit-display: -webkit-flex;
        display: -webkit-box;
        display: -ms-flexbox;
        display: flex;
        flex-direction: row;
        flex-wrap: nowrap;
        align-items: center;
        justify-content: flex-end;
      }
      .actions paper-button {
        flex: none;
      }
      h4 {
        font-size: 16px;
        font-weight: bold;
        margin: 2rem 0 0;
      }
      .editor-row {
        width: auto;
      }
      .editor-row.no-title {
        margin-top: 2rem;
      }
      .editor-row .old,
      .editor-row paper-input {
        width: 49%;
        display: inline-block;
        font-size:16px;
      }
      paper-button.delete_button {
        color: red;
      }
      paper-button.delete_button:hover {
        background: darkred;
      }
      paper-button.delete_button[disabled] {
        color: darkred;
      }


      @keyframes highlight-fade {
        0% {
          background: lightyellow;
        }
        100% {
          background: white;
        }
      }

      .old{
        font-family: monospace;
      }

      .highlight .old {
        animation: highlight-fade 1s ease-out;
      }

      paper-input{
        --paper-input-container: {
          padding:0;
        };

        --paper-input-container-input: {
          font-family:monospace;
        };
      }

      .material-icons {
        vertical-align: middle;
      }
    </style>

    <link href="https://fonts.googleapis.com/icon?family=Material+Icons" rel="stylesheet"/>

    <h4>Text</h4>
    <div class="editor-row">
      <span class="old">[[text.current]]</span>
      <paper-input class="input" id="textInput" value="{{text.value}}" name="text" label="Correction" error-message="Unable to create transcription correction from input" no-label-float=""></paper-input>
    </div>

    <h4>Standard</h4>
    <div class="editor-row">
      <span class="old">[[reg.current]]</span>
      <paper-input id="regInput" value="{{reg.value}}" name="reg" label="Regularise" no-label-float="" auto-validate=""></paper-input>
    </div>

    <div class="editor-row">
      <button on-click="toggle_morphology" type="button" class="btn btn-default btn-xs btn-block">
        <span> Morphology (optional)</span>
        <span id="morphology_span" class="material-icons">expand_more</span>
      </button>
    </div>

    <iron-collapse id="collapse_morphology" class="iron-collapse-closed">

    <h4>Lemma</h4>
    <div class="editor-row">
      <span class="old">[[lemma.current]]</span>
      <paper-input id="lemmaInput" value="{{lemma.value}}" name="lemma" label="Change Lemma" no-label-float="" auto-validate=""></paper-input>
    </div>

    <h4>Part of Speech (pos)</h4>
    <div class="editor-row">
      <span class="old">[[pos.current]]</span>
      <paper-input id="posInput" value="{{pos.value}}" label="Set part of speech" name="pos" no-label-float="" auto-validate=""></paper-input>
    </div>

    </iron-collapse>

    <div class="editor-row no-title">
      <paper-textarea id="noteInput" value="{{note.value}}" name="note" label="Add a note to your change set"></paper-textarea>
    </div>

    <div class="actions">
      <paper-button on-tap="cancel">Cancel</paper-button>
      <paper-button on-tap="handleApplyButtonTap" disabled\$="[[submitDisabled]]">Save</paper-button>
    </div>
`;
  }

  static get behaviors() {
    return Annotation.ActionTypes;
  }

  static get properties() {
    return {
      text: Object,
      lemma: Object,
      reg: Object,
      pos: Object,
      note: Object,
      allowedActions: {
        type: Array,
        value: function () { return [] }
      },
      fields: {
        type: Array,
        value: function () { return ['text', 'lemma', 'reg', 'pos'] }
      },
      disabled: {
        type: Boolean
      }
    }
  }

  connectedCallback() {
    super.connectedCallback();
    this.updateStyles();
  }

  ready() {
    super.ready();

    const textInput = this.shadowRoot.getElementById('textInput');
    textInput.addEventListener('change', event => this._textChanged(event));

    var inputs = ['regInput', 'lemmaInput', 'posInput', 'noteInput'];
    for (var index = 0; index < inputs.length; index++) {
      const input = this.shadowRoot.getElementById(inputs[index]);
      input.addEventListener('change', event => this._handleValueChange(event));
    }
    this.updateStyles();
  }

  setAllowedActions(allowedActions) {
    this.allowedActions = allowedActions;
  }

  setTargets(targets, annotations) {
    if (this.text.current === '') {
      this.set(['text.current'], targets.reduce(Annotation.textReducer, ''));
    }

    var mainTarget = targets[0];
    var reg;
    if (this.reg.current === '' && !('orig' in mainTarget)) {
      reg = mainTarget.original;
    }
    else if (this.reg.current === '' && 'orig' in mainTarget) {
      reg = mainTarget.getRegularized();
    }
    if (reg) {
      // Eliminate inner tagging such as <mark>.
      var elem = new DOMParser().parseFromString(reg, 'text/html');
      reg = elem.body.textContent || '';
      this.set(['reg.current'], reg);
    }

    this.targets = targets.map(function (element) {
      if ('orig' in element) { return element.orig }
      return element;
    })
    this.submitDisabled = true;

    if (this.lemma.current === '') {
      this.set(['lemma.current'], targets[0].getAttribute('lemma'));
    }
    if (this.pos.current === '') {
      this.set(['pos.current'], targets[0].getAttribute('pos'));
    }
  }

  cancel() {
    this._reset();
    this.dispatchEvent(new CustomEvent('cancelled', { bubbles: true, composed: true }));
  }

  handleApplyButtonTap(event) {
    event.stopPropagation();
    this.save();
  }

  // determine type of action from user input
  save() {
    if (!this.targets) {
      this._reset();
      this.dispatchEvent(new CustomEvent('closed', { bubbles: true, composed: true }));
    }

    var action = {
      ticket: Annotation.getNewSessionKey(),
      targets: this.targets
    }

    if (this.text.dirty && !this.$.textInput.invalid) {
      console.log('text');
      this.dispatchEvent(new CustomEvent('saved',
      {
        bubbles: true,
        composed: true,
        detail: {
          body: this.text.value,
          ticket: action.ticket,
          targets: action.targets,
          action: Annotation.ActionTypes.JOIN,
          id: this.text.annotationId
        }
      }));
    }

    if (this.reg.dirty && !this.$.regInput.invalid) {
      console.log('regularize');
      this.dispatchEvent(new CustomEvent('saved',
      {
        bubbles: true,
        composed: true,
        detail: {
          body: this.reg.value,
          ticket: action.ticket,
          targets: action.targets,
          attributeName: 'reg',
          action: Annotation.ActionTypes.UPDATE_ATTRIBUTE,
          id: this.reg.annotationId
        }
      }));
    }

    if (this.lemma.dirty && !this.$.lemmaInput.invalid) {
      console.log('lemma');
      this.dispatchEvent(new CustomEvent('saved',
      {
        bubbles: true,
        composed: true,
        detail : {
          body: this.lemma.value,
          ticket: action.ticket,
          targets: action.targets,
          attributeName: 'lemma',
          action: Annotation.ActionTypes.UPDATE_ATTRIBUTE,
          id: this.lemma.annotationId
        }
      }));
    }

    if (this.pos.dirty && !this.$.posInput.invalid) {
      console.log('pos');
      this.dispatchEvent(new CustomEvent('saved',
      {
        bubbles: true,
        composed: true,
        detail: {
          body: this.pos.value,
          ticket: action.ticket,
          targets: action.targets,
          attributeName: 'pos',
          action: Annotation.ActionTypes.UPDATE_ATTRIBUTE,
          id: this.pos.annotationId
        }
      }));
    }

    if (this.note.dirty) {
      console.log('add note');
      this.dispatchEvent(new CustomEvent('saved',
      {
        bubbles: true,
        composed: true,
        detail : {
          body: this.note.value,
          ticket: action.ticket,
          targets: action.targets,
          action: Annotation.ActionTypes.NOTE,
          id: this.note.annotationId
        }
      }));
    }

    this._reset();
    this.dispatchEvent(new CustomEvent('closed',
    {
      bubbles: true,
      composed: true,
      detail: {
        ticket: action.ticket,
        targets: action.targets
      }
    }));
  }

  _validateText(text) {
    return text.trim().length > 0;
  }

  // determine type of action from user input
  _textChanged(event) {
    this.$.textInput.invalid = !this._validateText(this.text.value);
    this.text.dirty = true;
    this._checkSubmitDisabled();
  }

  deleteWord(e) {
    var ticket = Annotation.getNewSessionKey();

    console.log('delete');
    this.dispatchEvent(new CustomEvent('saved', {
      bubbles: true,
      composed: true,
      detail: {
        ticket: ticket,
        body: '',
        targets: this.targets.slice(),
        action: Annotation.ActionTypes.DELETE
      }
    }));

    this._reset();
    this.dispatchEvent(new CustomEvent('editor.closed', {
      bubbles: true,
      composed: true}));
  }

  submit(e) {
    if (e.which === 13 || e.keyCode === 13) {
      this.save();
    }
  }

  getNewCleanValue(value) {
    return {
      value: value,
      current: '',
      dirty: false
    }
  }

  _reset() {
    this.$.textInput.invalid = false;
    this.text = this.getNewCleanValue('');
    this.reg = this.getNewCleanValue('');
    this.pos = this.getNewCleanValue('');
    this.lemma = this.getNewCleanValue('');
    this.note = this.getNewCleanValue('');
    this.defect = this.getNewCleanValue(false);
  }

  // observers
  _handleValueChange(event) {
    this[event.target.name].dirty = true;
    this._checkSubmitDisabled();
  }

  _checkSubmitDisabled() {
    this.submitDisabled = !(
      this.text.action === '' ||
      this.text.dirty ||
      this.reg.dirty ||
      this.pos.dirty ||
      this.lemma.dirty
    ) ||
      this.$.textInput.invalid ||
      this.$.regInput.invalid ||
      this.$.posInput.invalid ||
      this.$.lemmaInput.invalid
  }

  toggle_morphology() {
    var e = this.shadowRoot.getElementById("morphology_span");
    if (this.$.collapse_morphology.opened) {
      e.innerHTML = 'expand_more';
    }
    else {
      e.innerHTML = "expand_less";
    }
    this.$.collapse_morphology.toggle();
  }
}

customElements.define('join-editor', JoinEditor);
