import { PolymerElement, html } from '@polymer/polymer/polymer-element.js';
import { AtomicNode } from './atomic-node.js';
import { Annotation } from './annotation.js';
/*
  <tei-choice>
    <tei-orig><tei-w id='o1'>foo</tei-w></tei-orig>
    <tei-reg><tei-w id='r1'>foo</tei-w></tei-reg>
  </tei-choice>
*/

var nodeNameToType = {
  'TEI-REG': 'regularization',
  'TEI-ORIG': 'regularization'
};

class TeiChoice extends AtomicNode {
  static get is() {
    return 'tei-choice';
  }
  static get properties() {
    return {
      choiceType: {
        type: String,
        reflectToAttribute: true,
        value: 'unkown'
      },
      displayId: {
        type: String,
        reflectToAttribute: true,
        value: '',
        observer: '_handleDisplayIdChange'
      },
      displayType: {
        type: String,
        reflectToAttribute: true,
        value: 'orig',
        observer: '_handleDisplayTypeChange'
      }
    }
  }

  static get behaviors() {
    return [
      Annotation.itemSelectable
    ];
  }

  connectedCallback() {
    super.connectedCallback();
    this.choices = this.children;
    for (var l = this.choices.length; --l;) {
      this.choices[l].hidden = true;
    }
    var firstElement = this.choices[0];
    this.orig = document.querySelector('tei-orig tei-w');
    this.orig.addEventListener('bg.change', event => this._changeBg(event));
    this.reg = document.querySelector('tei-reg tei-w');

    if (!firstElement) {
      console.warn('TEI-CHOICE error: no choices found');
      return;
    }
    // determine type of choice
    var type = nodeNameToType[firstElement.nodeName];
    if (!type) {
      console.warn('TEI-CHOICE: unsupported choice type');
      this.choiceType = 'unknown';
      return;
    }
    this.choiceType = type;
  }

  disconnectedCallback() {
    super.disconnectedCallback();
    this.orig.removeEventListener('bg.change', event => this._changeBg(event));
  }

  getOriginal() {
    if (this.choiceType === 'regularization') {
      return this.orig.firstChild.textContent.trim();
    }
    return null;
  }

  getRegularized() {
    if (this.choiceType === 'regularization') {
      return this.reg.firstChild.textContent.trim();
    }
    return null;
  }

  getOrigAttribute(attributeName) {
    if (this.choiceType === 'regularization') {
      var w = document.querySelector('tei-orig tei-w');
      return w.getAttribute(attributeName);
    }
    return null;
  }

  setRegularized(regularForm) {
    if (this.choiceType === 'regularization') {
      return this.reg.firstChild.textContent = regularForm.trim();
    }
    return null;
  }

  _handleDisplayIdChange(newVal, oldVal) {
    if (!newVal || !newVal.length || (newVal === oldVal)) { return }
    // look at the new id and find the corresponding element

    var selectedElement = document.querySelector('#' + newVal);
    if (!selectedElement) { return }

    var currentElement = document.querySelector('#' + newVal);
    if (currentElement) {
      currentElement.parentElement.hidden = true;
    }

    this._setDisplay(selectedElement.parentElement);
  }

  _handleDisplayTypeChange(newVal, oldVal) {
    if (!newVal || !newVal.length || (newVal === oldVal)) { return }

    var selectedElement = document.querySelector('tei-' + newVal);
    if (!selectedElement) { return }

    var currentElement = document.querySelector('tei-' + oldVal);
    if (currentElement) {
      currentElement.hidden = true;
    }
    this._setDisplay(selectedElement);
  }

  _setDisplay(selectedElement) {
    var newType = selectedElement.nodeName.substring(4).toLowerCase();
    this.set('displayType', newType);
    this.set('displayId', selectedElement.firstChild.id);
    selectedElement.hidden = false;
  }

  _changeBg(newBgEvent) {
    this.reg.style.backgroundColor = newBgEvent.detail;
  }
}

customElements.define(TeiChoice.is, TeiChoice);