import { PolymerElement, html } from '@polymer/polymer/polymer-element.js';
import {} from '@polymer/polymer/lib/elements/dom-repeat.js';

import './annotation-list-item.js';

class TicketListItem extends PolymerElement {
  static get template() {
    return html`
    <style>

    h5,
    annotation-list-item {
      display: block;
      margin: 1rem 2rem;
    }

    .user-info {
      font-size: 1.25rem;
      margin: 1rem 2rem;
    }

    .actions {
      margin-top: 2rem;
      -webkit-display: -webkit-box;
      -webkit-display: -webkit-flex;
      display: -webkit-box;
      display: -ms-flexbox;
      display: flex;
      flex-direction: row;
      flex-wrap: nowrap;
      align-items: center;
      justify-content: flex-end;
      width:100%;
      background-color: #eee;
    }
    .actions paper-button {
      flex: none;
    }
    .actions paper-button[active] {
      color:#CA3227;
    }
    .actions paper-button[disabled] {
      background-color: #eee;
    }

    iron-collapse {
      margin: 0 2rem 2rem;
    }
    paper-button.confirm{
      display: inline-block;
      float:right;
      margin-top:-38px;
      font-size:12px;
    }
    </style>

    <div class="user-info">
      <span class="user">by [[item.annotations.0.creator]]</span>
      <span class="date">on [[formatDate(item.annotations)]]</span>
      <span class="status">([[formatStatus(item.status)]])</span>
    </div>
    <template is="dom-if" if="[[hasSplitAnnotation]]">
      <annotation-list-item item="[[splitAnnotation]]">
    </annotation-list-item></template>
    <template id="annotationlist" is="dom-repeat" items="[[item.annotations]]" filter="notTargetingANewWord">
      <annotation-list-item item="[[item]]">
    </annotation-list-item></template>
    <template id="annotationlist2" is="dom-repeat" items="[[item.annotations]]" filter="targettingANewWord">
      <annotation-list-item item="[[item]]">
    </annotation-list-item></template>
    <template is="dom-if" if="[[showActions(item.status, user.role)]]">
    <div class="actions">
      <paper-button class="edit" on-tap="edit" disabled\$="[[editingDisabled(item.status, item.creator)]]">edit</paper-button>
      <paper-button class="delete" on-tap="delete" disabled\$="[[editingDisabled(item.status, item.creator)]]">delete</paper-button>
      <paper-button class="approve" on-tap="approve" disabled\$="[[isDisabled(item.status)]]" hidden\$="[[isNormalUser(user.role)]]">approve</paper-button>
      <paper-button class="reject" on-tap="openReasonInput" toggles="" disabled\$="[[isDisabled(item.status)]]" hidden\$="[[isNormalUser(user.role)]]">reject</paper-button>
    </div>
    </template>
    <iron-collapse id="reasonInputContainer">
      <paper-input id="reasonInput" label="reason" tabindex="0" required="" auto-validate="" error-message="Please enter a reason">
      </paper-input>
      <paper-button class="confirm" on-tap="reject">confirm</paper-button>
    </iron-collapse>
`;
  }

  static get properties() {
    return {
      item: {
        type: Object,
        notify: true,
        observer: 'itemChanged'
      },
      user: Object,
      splitAnnotation: {
        type: Object,
        computed: 'getSplitAnnotation(item.annotations)'
      },
      hasSplitAnnotation: {
        type: Boolean,
        computed: 'checkSplitAnnotation(splitAnnotation)',
        value: false
      }
    }
  }

  // Don't leave open when moving to a new word.
  // actions
  itemChanged(newVal, oldVal) {
    if (!this.$.reasonInputContainer.opened) { return }
    this.$.reasonInputContainer.toggle();
  }

  edit(event, detail) {
    this.dispatchEvent(new CustomEvent('edit', { bubbles: true, composed: true }));
  }

  delete(event, detail) {
    if (confirm('Really delete?') == false) return;
    this.dispatchEvent(new CustomEvent('delete', { bubbles: true, composed: true }));
  }

  approve(event, detail) {
    this.dispatchEvent(new CustomEvent('approve', { bubbles: true, composed: true }));
  }

  openReasonInput(event) {
    if (this.$.reasonInputContainer.opened) { return } // already open
    this.$.reasonInputContainer.toggle();
    this.$.reasonInput.focus();
  }

  reject(event, detail) {
    var reason = this.$.reasonInput.value;
    if (!reason || reason.value === '') { return }
    this.$.reasonInputContainer.toggle();
    this.dispatchEvent(new CustomEvent('reject', {
      bubbles: true,
      composed: true,
        detail: {
          reason: reason
        }
    }));
    this.$.reasonInput.value = ''
  }

  // computed properties

  getSplitAnnotation(annotations) {
    if (!annotations) return null;
    return annotations.reduce(function (result, annotation) {
      if (annotation.type === 'split') { return annotation }
      return result;
    }, null);
  }

  checkSplitAnnotation(splitAnnotation, b) {
    console.log('checkSplitAnnotation', splitAnnotation);
    return (splitAnnotation && 'id' in splitAnnotation);
  }

  editingDisabled(status, creator) {
    return (status === 'accepted' ||
      creator !== this.user.name);
  }

  showActions(status, role) {
    return (status !== 'accepted' && role !== "reader");
  }

  isDisabled(isNew, status) {
    return (status === 'accepted');
  }

  isNormalUser(role) {
    return (role !== 'shcadmin');
  }

  // formatters

  formatStatus(status) {
    switch (status) {
      case 'pending': return 'approval pending';
      case 'accepted': return 'approved';
      case 'rejected': return 'rejected';
      case 'saved': return 'saved';
      default: return 'unkown';
    }
  }

  formatDate(annotations) {
    var date = annotations[0].modifiedDate;
    if (!date) { return '' }
    return date.toDateString();
  }

  visibilityIcon(appliedState) {
    if (appliedState) {
      return 'visibility';
    }
    return 'visibility-off';
  }

  // filters

  notTargetingANewWord(annotation) {
    return (annotation.type !== 'split' &&
      !this.targettingANewWord(annotation));
  }

  targettingANewWord(annotation) {
    var selector = annotation.target.limits[0].getAttribute('value');
    return (/-s-3$/.test(selector));
  }
}

customElements.define('ticket-list-item', TicketListItem);
