var localData;
var currentPage;

function getLocalData() {
    return localData;
}

function getCurrentPage() {
    return currentPage;
}

function transform(data) {
    var result = {
        "hits": data.summary.numberOfHits,
        "matches": []
    };

    for (var j = 0; j < data.hits.length; j++) {
        var hit = data.hits[j];
        var match = {
            "match": {
                "words": hit.match.id.map(function(id){
                                                return data.docInfos[hit.docPid].idno[0].concat('-', id);
                                            })
            },
            "page": [ hit.match.id[0].substr(0, hit.match.id[0].lastIndexOf('-')) ]
        }
        result.matches.push(match);
    }
    return result;
}

async function loadKwicMatches(doc, pattern, matchParam) {
    const target = 'https://eplab.artsci.wustl.edu/blacklab-server-1.7.3/eebotcp/hits?outputformat=json&number=15000&wordsaroundhit=0&patt=' + encodeURIComponent(pattern) + '&filter=idno:' + doc;

    if(localData){
        return markAllMatches(localData, matchParam, pattern);
    }

    console.log ('blacklab hits url:', target);
    await fetch(target, {
        method: 'GET',
        mode: 'cors',
        credentials: 'same-origin'
    })
        .then((response) => response.json())
        .then((blacklabData) => {
            data = transform(blacklabData);
            localData = data;
            markAllMatches(data, matchParam, pattern);
        })
}

function markAllMatches(kwicData, matchParam, pattern) {

    if(!pattern) {return};

    const count = kwicData.hits;
    jQuery('#count').text(count);
    let currentIdx = 0;

    var hits = kwicData.matches;
    let calcMatch;

    // if there's no matchParam we're opened by document link at use first match as default
    if(matchParam){
        calcMatch = matchParam;
    }
    else{
        if(Array.isArray(hits)){
            calcMatch = hits[0].match.words[0];
        }
        else{
            calcMatch = hits.match.words[0];
        }
    }

    if (Array.isArray(hits)) {
        hits.forEach((hit,index) => {
            let className = 'kwic-highlight';
            if (hit.match.words[0] === calcMatch){
                className = 'kwic-current' ;
                jQuery('#current').text(index);
                currentIdx = index;
                currentPage = hit;
                currentElement = document.getElementById(calcMatch);
            }
            hit.match.words.forEach((word) => {
                addMarkerClass(word, className);
            });
        });
    }
    else {
        // ### it's just a single hit and we get object instead of array
        let className = 'kwic-highlight';
        if (hits.match.words[0] === calcMatch) className = 'kwic-current';

        hits.match.words.forEach((word) => {
            addMarkerClass(word, className);
        });
    }

    jQuery('#current').text(currentIdx+1);

    const pathArray = window.location.pathname.split('/');
    const last = pathArray.length - 1
    setMatchNavigation(hits, calcMatch, pattern, pathArray[last], currentIdx);
}

function addMarkerClass(wordId, className) {
    let el = document.querySelector(`#${wordId}`);
    if (!el) {
        return;
    }
    el.classList.add(className);
}

function setMatchNavigation(hits, current, pattern, doc, index) {
    var link = doc + '?';

    const matchElem = document.getElementById(current);
    if(!matchElem) return;
    matchElem.scrollIntoView({block: "center", inline: "nearest"});

    const actual = index;

    if (actual > 0) {
        let prevKey = actual - 1;
        let prevPage =  hits[prevKey].page[0];
        let prevMatch = hits[prevKey].match.words[0];
        setSearchNavigationButtonState(jQuery('.nav-match-prev'), link + 'page=' + prevPage, pattern, prevMatch,prevPage);
    }

    if (actual < hits.length) {
        nextKey = actual + 1
        nextPage = hits[nextKey].page[0];
        nextMatch = hits[nextKey].match.words[0];
        setSearchNavigationButtonState(jQuery('.nav-match-next'), link + 'page=' + nextPage, pattern, nextMatch, nextPage);
    }

    jQuery('#current').text(actual+1);
}


function setSearchNavigationButtonState(button, link, pattern, match,pageId) {
    var params = '';
    if (pattern) {
        params += '&pattern=' + pattern;
    }
    if (match) {
        params += '&match=' + match;
    }
    if (!link) {
        return button.removeAttr('href').attr("disabled", true);
    }
    button.attr("href", link+params).removeAttr("disabled");
    button.attr('data-nav',match);
    button.attr('data-page',pageId);
    button.attr('data-pattern',pattern);
}

function setMatchCounterState(id, index) {
    label = document.getElementById(id)
    label.innerHTML = index + ' of ';
}


